// **********************************************************************
//
//                          EXIMDL
//
// This program solves the following problem:
//
// Maximize   4x1 - 2x2 + 7x3 - x4
//
// Subject to:
//
//  x1         +  5x3        <= 10
//  x1 +   x2  -   x3        <=  1
// 6x1 -  5x2                <=  0
// -x1         +  2x3 -  2x4 <=  3
//
//
// And subject to:
//
// 0 <= x1 <= 1.0D+6 ; x1 integer variable
// 0 <= x2 <= 1.0D+6 ; x2 integer variable
// 0 <= x3 <= 1.0D+6 ; x3 integer variable
// 0 <= x4 <= 1.0D+6
//
// NROW     is the number of rows in the constraint matrix.
// NCOL     is the number of columns in the constraint matrix.
// NEL      is the number of elements in the constraint matrix.
// ITYPE    is the storage format.
// IRL      is the length of the arrays containing row information.
// ICL      is the length of the arrays containing column information.
// ICL1     is ICL + 1
// NINTS    is the number of integer variables.
// MINTS    is the variable numbers of integer variables.
// NSETS    is the number of sets of integer variables.
// NTSIZE   is the total size of sets plus variables.
// NSSETS   is the variable numbers of variables in sets.
// IMDLTP   is the types of sets of integer variables.
//
// *******************************************************************

#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include "ekk_c_api.h"


// Model dimensions 

#define NROW   4
#define NCOL   4
#define NEL   10
#define NSETS 3

// Matrix data

int     ia[NEL] = {0,1,2,3,1,2,0,1,3,3};               // Row indices        
int     ja[NEL] = {0,0,0,0,1,1,2,2,2,3};               // Column indices     
double  drlo[NROW] = {-1.0e31,-1.0e31,-1.0e31,-1.0e31};// Row lower bounds   
double  drup[NROW] = {10.0, 1.0, 0.0, 3.0};            // Row upper bounds   
double  dclo[NCOL] = {0.0, 0.0, 0.0, 0.0};             // Column lower bounds
double  dcup[NCOL] = {1.0e6, 1.0e6, 1.0e6, 1.0e6};     // Column upper bounds
double  dobj[NCOL] = {4.0, -2.0, 7.0, -1.0};           // Objective function 
double  dels[NEL] = {1.0, 1.0, 6.0, -1.0, 1.0, -5.0, 5.0, -1.0, 2.0, -2.0}; 
                                                       // Matrix elements    

// Integer Data -- 3 integers in 3 sets;  1 in each set

int     set1ints[1] = {0};                     // Column 0 in set 1          
int     set2ints[1] = {1};                     // Column 1 in set 2          
int     set3ints[1] = {2};                     // Column 2 in set 3          
double  set1dnpcost[1] = {5.0e-3};             // Down pseudo-costs for set 1
double  set1uppcost[1] = {5.0e-3};             // Up pseudo-costs for set 1  
double  set2dnpcost[1] = {5.0e-3};             // Down pseudo-costsfor set 2 
double  set2uppcost[1] = {5.0e-3};             // Up pseudo-costs for set 2  
double  set3dnpcost[1] = {5.0e-3};             // Down pseudo-costs for set 3
double  set3uppcost[1] = {5.0e-3};             // Up pseudo-costs for set 3  

void main()
{
    // Initialize OSL environment 
    EKKContext * env;
    env = ekk_initializeContext();

    // Create a model             
    EKKModel * mipmodel;
    mipmodel=ekk_newModel(env,NULL);

    // Set to print parameter value upon entry to OSL functions
    ekk_setPrinting(mipmodel,1,1,0);

    // Set as a maximization problem
    int rtcod = ekk_setRmaxmin(mipmodel,-1.000000);

    // Load row/column bounds, obj function
    rtcod = ekk_loadRimModel(mipmodel,NROW,drlo,drup,NCOL,dobj,dclo,dcup);

    // Delete the last block      
    rtcod = ekk_dropLastElementBlock(mipmodel);

     // Add matrix elements to model as block
    rtcod = ekk_addElementBlock(mipmodel, NEL, ia, ja, dels);

    // Merge all blocks into 1 block
    rtcod = ekk_mergeBlocks(mipmodel,1);

    // Add integer set 1 
    rtcod = ekk_addIntegerSet(mipmodel, 1000, 1, set1ints, set1dnpcost, set1uppcost);

    // Add integer set 2
    rtcod = ekk_addIntegerSet(mipmodel, 1000, 1, set2ints, set2dnpcost, set2uppcost);

    // Add integer set 3          
    rtcod = ekk_addIntegerSet(mipmodel, 1000, 1, set3ints, set3dnpcost, set3uppcost);

    // Solve the LP relaxation by simplex
    rtcod = ekk_eitherSimplex(mipmodel);

    // Solve by branch-and-bound  
    rtcod = ekk_branchAndBound(mipmodel,NULL,NULL);

    // Print solution             
    rtcod = ekk_printSolution(mipmodel);

    // Destroy the model          
    rtcod = ekk_deleteModel(mipmodel);

    // OSL Destructor             
    ekk_endContext(env);

}
