// *******************************************************************
//
//                          EXLPDC
//
// This program demonstrates one use of ekk_decomposeCrash. A problem stored
// in MPS format is read in by calling ekk_importModel. Next, a call is
// made to ekk_decomposeCrash to decompose the problem into a master problem
// with subproblems, and to do one sweep of Dantzig-Wolfe decomposition
// to find an advanced basis. Finally, ekk_primalSimplex is called to
// solve the problem.
//
// ********************************************************************

#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include "ekk_c_api.h"

void main()
{
  // Initialize OSL environment    
  EKKContext * env;
  env = ekk_initializeContext();           
  
  // Create an OSL model "test"
  EKKModel * test;
  test = ekk_newModel(env,NULL);             
  
  // Read model data from MPS file 
  int rtcod = ekk_importModel(test,"exlp6.mps"); 
  
  // Write model data to file in binary mode
  rtcod = ekk_saveModel(test,"exlp6.ptmd");  
  
  //
  //  Decompose the problem and get an advanced basis.
  //  type=2     : one pass of Dantzig-Wolfe decomposition.
  //  strategy=1 : do not include master rows as constr. for subproblems.
  //  nblocks=3  : decompose the matrix into 3 blocks. Note that if
  //             nblocks=0, OSL determines the number of blocks.
  //
  
  // Decompose the problem and get an advanced basis.           
  rtcod = ekk_decomposeCrash(test,2,1,3);     
  
  // Solve using simplex method   
  rtcod = ekk_primalSimplex(test,0);          
  
  // Set to print column solutions only               
  rtcod = ekk_setIsolmask(test,6);            
  
  // Print the solution           
  rtcod = ekk_printSolution(test);            
  
  // Read model data from file in binary mode             
  rtcod = ekk_restoreModel(test,"exlp6.ptmd");
  
  // Allocate a row status vector        
  int * rowstat = new int[ekk_getInumrows(test)];         
  
  // Get row names  
  const char * const * rowNames = ekk_rowNamePointers(test); 

  // Set rowstat based on row name
  for( int irow = 0; irow < ekk_getInumrows(test); irow++) {
    rowstat[irow]=0;
    
    // if row name begins with "B01", then set row status to 1
    if (!strncmp("B01",rowNames[irow],3)) 
      rowstat[irow]=1;
    
    // if row name begins with "B02", then set row status to 2
    if (!strncmp("B02",rowNames[irow],3)) 
      rowstat[irow]=2;
    
    // if row name begins with "B03", then set row status to 3
    if (!strncmp("B03",rowNames[irow],3)) 
      rowstat[irow]=3;
  }
  
  // Set row status values
  ekk_copyRowstat(test,rowstat,0,ekk_getInumrows(test));
  
  //
  //  Decompose the problem and get an advanced basis.
  //  type=2     : one pass of Dantzig-Wolfe decomposition.
  //  strategy=1 : OSL solves each decomposed subproblem independently.
  //  nblocks=-1 : user has placed block numbers in the row status region.
  //
  
  // Decompose the problem and get an advanced basis.      
  rtcod = ekk_decomposeCrash(test,2,1,-1);   
  
  // Solve using simplex method    
  rtcod = ekk_primalSimplex(test,0);         
  
  // Set to print column solutions only              
  rtcod = ekk_setIsolmask(test,6);           
  
  // Print the solution            
  rtcod = ekk_printSolution(test);           
  
  // Read model data from file in binary mode            
  rtcod = ekk_restoreModel(test,"exlp6.ptmd");            

  // Get row names  
  rowNames = ekk_rowNamePointers(test);
   
  // Set rowstat based on row name
  for(irow = 0; irow < ekk_getInumrows(test); irow++) {
    rowstat[irow]=0;
    
    // if row name begins with "B01", then set row status to 1
    if (!strncmp("B01",rowNames[irow],3)) 
      rowstat[irow]=1;
    
    // if row name begins with "B02", then set row status to 2
    if (!strncmp("B02",rowNames[irow],3)) 
      rowstat[irow]=2;
    
    // if row name begins with "B03", then set row status to 3
    if (!strncmp("B03",rowNames[irow],3)) 
      rowstat[irow]=3;
  }
  
  // Set row status values
  ekk_copyRowstat(test,rowstat,0,ekk_getInumrows(test));
  
  //
  //  Decompose the problem and get an advanced basis.
  //  type=2     : one pass of Dantzig-Wolfe decomposition.
  //  strategy=2 : OSL partitions the RHS for each subproblem.
  //  nblocks=-1 : user has placed block numbers in the row status region.
  //
  
  // Decompose the problem and get an advanced basis.        
  rtcod = ekk_decomposeCrash(test,2,2,-1);   
  
  // Solve using simplex method    
  rtcod = ekk_primalSimplex(test,0);         
  
  // Set to print column solutions only                
  rtcod = ekk_setIsolmask(test,6);           
  
  // Print the solution            
  rtcod = ekk_printSolution(test);           
  
  // Free allocated storage
  delete [] rowstat;
  
  // Delete the model              
  rtcod = ekk_deleteModel(test); 
  
  // OSL Destructor                
  ekk_endContext(env);                     
}
