// **********************************************************************
//
//                           EXLPDUAL
//
//  This program reads a minimization problem from an MPS file, sets
//  the appropriate control variable to set to print primal/dual solution
//  vectors at optimality, solves the problem with the simplex method,
//  and prints the solution.
//
// **********************************************************************
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include "ekk_c_api.h"

#define __fabs(x) ((x) < 0 ? (-x): (x)) // so XlC can inline fabs

void eval(EKKModel *model);

void main()
{
    // Initialize OSL environment      
    EKKContext * env;
    env = ekk_initializeContext();

    // Create an OSL model "test"      
    EKKModel * test;
    test = ekk_newModel(env,NULL);

    // Read model data from MPS file 
    int rtcod=ekk_importModel(test,"exlp10.mps");

    // Scale the model data           
    rtcod = ekk_scale(test);

    // Solve using simplex method     
    rtcod = ekk_primalSimplex(test,2);

    // Set to print primal/dual solutions
    ekk_setIprtinfomask(test,63);

    // Print the solution             
    rtcod = ekk_printSolution(test);
  
    // Verify primal obj = dual obj   
    eval(test);
  
    // Delete the model               
    rtcod = ekk_deleteModel(test);
  
  
    // OSL Destructor                 
    ekk_endContext(env);
}

// **********************************************************************
//    This routine evaluates the primal and dual objective functions.
// **********************************************************************
  

void eval(EKKModel *model){
  
    // Obtain address of objective function vector
    const double *cost = ekk_objective(model);

    // Obtain address of column solution vector
    const double *soln = ekk_colsol(model);

    // Obtain address of row activities vector
    const double *ract = ekk_rowacts(model);

    // Obtain address of column lower bounds vector
    const double *dclo = ekk_collower(model);

    // Obtain address of column upper bounds vector
    const double *dcup = ekk_colupper(model);

    // Obtain address of row lower bounds vector
    const double *drlo = ekk_rowlower(model);

    // Obtain address of row upper bounds vector
    const double *drup = ekk_rowupper(model);

    // Obtain address of column reduced costs vector
    const double *cdual = ekk_colrcosts(model);

    // Obtain address of row duals vector -- row duals are of the opposite 
    // sign as those from the older interface
    const double *rdual = ekk_rowduals(model);

    // Obtain problem name
    const char *CNAME = ekk_getCname(model);

    // Obtain number of rows in the model
    long INUMROWS = ekk_getInumrows(model);

    // Obtain number of columns in the model
    long INUMCOLS = ekk_getInumcols(model);


    double zeps = 0.00000001e0, zdiff;

    double primal = 0.0e0;

    // Calculate primal objective function.
    for (int i = 0; i<INUMCOLS; i++){
         primal += cost[i]*soln[i];
    }

    // Calculate dual objective function.
    double dual = 0.0e0;

    // Compute the inner-product with the row duals.
    for (i = 0; i<INUMCOLS; i++) {
        zdiff = fabs(dclo[i]-dcup[i]);

        // Check if the variable is fixed. 
        if (zdiff <  2*zeps) {
             dual += cdual[i]*dcup[i];
        } else {
             zdiff = fabs(soln[i]-dclo[i]);

             // Check if the variable is at lower bound. 
             if (zdiff < zeps)
                dual += cdual[i]*dclo[i];

             zdiff = fabs(soln[i]-dcup[i]);

             // Check if the variable is at upper bound (and corr. dual is 
             // negated). 
             if (zdiff < zeps)
                dual += cdual[i]*dcup[i];
        }
     }

     // Compute the inner-product with the reduced costs.
     for (i = 0; i<INUMROWS; i++) {
         zdiff = fabs(drup[i]-drlo[i]);

         // Check if the constraint is an equality.  
         if (zdiff <  2*zeps) {
            dual += rdual[i]*drup[i];
         } else {
            zdiff = fabs(ract[i]-drlo[i]);

            if (zdiff < zeps)
                // Check if the row activity is at lower bound
                dual += rdual[i]*drlo[i];

            zdiff = fabs(ract[i]-drup[i]);

            // Check if the row activity is at upper bound (and corr. dual is negated)
            if (zdiff < zeps)
               dual += rdual[i]*drup[i];
          }
      }

     printf("\n  *****  Name  %s   Primal   %f   Dual  %f *****\n", CNAME,primal,dual);
}
