//  ------------------------------------------------------------------
//                          EXMIP
//  This program reads a maximization problem from an MPS file, sets
//  the appropriate real control variable to indicate that it is a
//  maximization problem, sets the appropriate integer control
//  variable required for branch and bound preprocessing, calls the
//  branch and bound preprocessing routine, then solves the problem
//  with the branch and bound method, and lastly prints the results.
// --------------------------------------------------------------------

#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include "ekkc.h"
#include "ekk_c_api.h"

// MIP Test
void main()
{
  // Initialize OSL environment
  EKKContext * env;
  env = ekk_initializeContext();
  
  // Create a model           
  EKKModel * mipModel;
  mipModel=ekk_newModel(env,NULL);
  
  // Set line length to 150 characters
  int rtcod = ekk_setIlinelen(mipModel,150);
  
  // Set to print primal and dual solutions
  rtcod = ekk_setIprtinfomask(mipModel,63);
  
  // Set to print the column solutions only
  rtcod = ekk_setIsolmask(mipModel,6);
  
  // Set maximum allowable iteration limit to 9999  
  rtcod = ekk_setImaxiter(mipModel,9999);
  
  // Set ItruCallBack usage frequency to 9999999     
  rtcod = ekk_setIiterufreq(mipModel,9999999);
  
  // Set to print the parameter values for the OSL functions
  ekk_setPrinting(mipModel,1,1,0);
  
  // Set as a maximization problem
  rtcod = ekk_setRmaxmin(mipModel,-1.000000);
  
  // Read model data from MPS file
  rtcod = ekk_importModel(mipModel,"exmip1.mps");
  
  // Increase maximum allowable rows by 1000
  rtcod = ekk_setImaxrows(mipModel,ekk_getInumrows(mipModel)+1000);
  
  // Set to compress matrix after presolve
  ekk_setCompressOn(mipModel);
  
  // Set maximum allowable solutions (to be generated) to 999999
  rtcod = ekk_setImaxsols(mipModel,999999);
  
  // Set maximum allowable nodes (to be generated) limit to 9999999
  rtcod = ekk_setImaxnodes(mipModel,9999999);
  
  // Set Istrategy to 1       
  rtcod = ekk_setIstrategy(mipModel,1);
  
  // Set number of heuristic passes to 1
  rtcod = ekk_setIheurpass(mipModel,1);
  
  // Presolve the model       
  rtcod = ekk_preSolve(mipModel,3,"presolve3");
  
  // Create a starting basis  
  rtcod = ekk_crash(mipModel,1);
  
  // Solve model by simplex   
  rtcod = ekk_eitherSimplex(mipModel);
  
#if 0
  // Create work space to be able to make V2 type calls   
  double * dspace = (double*)malloc(3000000*sizeof(double));
  ekk_loadDspace(mipModel, dspace, 3000000);
  
  // Preprocess the 0/1 model 
  long rc;
  ekkmpre(&rc,dspace,2);
  
  // Solve by branch-and-bound
  ekkmslv(&rc,dspace,1,0,0);
  
  // Copy data from V2 work space to V3 object and free V2 work space
  rtcod = ekk_unloadDspace(mipModel,dspace);
  free(dspace);
#else
  // Preporcess the MIP
  rtcod = ekk_integerPresolve(mipModel,NULL,NULL,0);

  // Solve by branch and bound
  rtcod = ekk_branchAndBound(mipModel,NULL,NULL);
#endif
  
  // Postsolve the model      
  rtcod = ekk_postSolve(mipModel,"presolve3");
  remove("presolve3");
  
  // Get column solutions     
  const double* colsol = ekk_colsol(mipModel);
  
  // Get column lower bounds  
  double* collower = ekk_getCollower(mipModel);
  
  // Get column upper bounds  
  double* colupper = ekk_getColupper(mipModel);
  
  // Get number of integers   
  int  numints = ekk_getInumints(mipModel);
  
  // Get the list of integers from the model
  int * intinfoSave = ekk_listOfIntegers(mipModel);
  
  for ( int i=0; i<numints; i++ ) {       
    // Fix lower bounds at solution value
    collower[intinfoSave[i]] = colsol[intinfoSave[i]]; 
    // Fix upper bounds at solution value
    colupper[intinfoSave[i]] = colsol[intinfoSave[i]];
  }
  int nc = ekk_getInumcols(mipModel);
  ekk_copyCollower(mipModel,collower,0,nc );
  ekk_copyColupper(mipModel,colupper,0,nc );
  
  // Free space 
  ekk_free(intinfoSave);
  ekk_free(colupper);
  ekk_free(collower);
  
  // Set printing off for all messages
  rtcod = ekk_messagesPrintOff(mipModel,1,9999);
  
  // Solve by simplex         
  rtcod = ekk_primalSimplex(mipModel,0);  
  
  // Set printing on for all messages
  rtcod=ekk_messagesPrintOn(mipModel,1,9999);
  
  // Print solution           
  rtcod = ekk_printSolution(mipModel);
  
  // Destroy the model        
  rtcod = ekk_deleteModel(mipModel);
  
  // OSL Destructor           
  ekk_endContext(env);
}
