// --------------------------------------------------------------------
//                           EXRPTB
//  This program solves the following maximization problem:
//  Maximize  4*x4 + 0.1*x5 + 6*x10 + 0.15*x11 + 8*x16 + 0.15*x17
//              + 6*x22 + 0.15*x23 + 3*x26 + 0.1*x27
//              + 3*x29 + 0.1*x30
//  Subject to:  Dx  = b; x >= 0
//  where trans(b) = (100,50,0,0,0,0,0,0,0,0,0,0)
//  and the matrix D has the form:
//     --------
//     |  A   |
//     |      |
//     -------+-------
//     |      |  A   |
//     |  B   |      |
//     |      |------+-------
//     |      |      |  A   |
//     -------|  B   |      |
//            |      |------+-------
//            |      |      |  A   |
//            -------|  B   |      |
//                   |      |------+---------
//                   |      |      |        |
//                   -------|  B   |    C   |
//                          |      |        |
//                          |      |        |
//                          -----------------
//  A is a 2 X 6 matrix stored by indices.
//             __                     __
//             | 1   1   1   1   0   0 |
//         A = |                       |
//             | 3   0   0   0   1   1 |
//             --                     --
//  B is a 4 X 6 matrix stored by columns.
//              __                    __
//             | 0   0  -1   0   0   0 |
//             |                       |
//             | 0   0   0   0   0  -1 |
//         B = |                       |
//             |-4  -1   0   0   0   0 |
//             |                       |
//             | 0 -10   0   0   0   0 |
//             --                     --
//  C is a 4 X 6 matrix stored by indices.
//              __                    __
//             | 1   1   0   0   0   0 |
//             |                       |
//             | 0   0   1   1   0   0 |
//         C = |                       |
//             |-1   0   0   0   1   0 |
//             |                       |
//             | 0   0   0  -1   0   1 |
//             --                     --
//
//   Illustrates the use of ekk_repeatElementBlock to input the blocks
//   A, and B, multiple times.
// ---------------------------------------------------------------------

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include "ekk_c_api.h"


void main()
{
    // Initialize OSL environment   
    EKKContext * env;
    env = ekk_initializeContext();

    // Create an OSL model "test"   
    EKKModel * test;
    test = ekk_newModel(env,NULL);

    // Set to maximization problem   
    int rtcod = ekk_setMaximize(test);

    // ----------------------------------------------
    // Define Rim Data
    // ---------------------------------------------- 
    // total no. of rows in the composite matrix 
    const int NROW = 12; 
    // total no. of cols in the composite matrix 
    const int NCOL = 30;     
    // Lower & Upper bounds on row activities   
    double rowRhs[NROW] = { 100.,50.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.};
    // Obj. func. coeff. on columns   
    double obj[NCOL] = { 0.,0.,0.,4.,.1,0.,0.,0.,0.,6.,.15,0.,0.,0.,
        0.,8.,.15,0.,0.,0.,0.,6.,.15,0.,0.,3.,.1,0.,3.,.1 };
    // ----------------------------------------------

    // Set the rim vectors of the model   
    rtcod = ekk_loadRimModel(test,NROW,rowRhs,rowRhs,NCOL,obj,NULL,NULL);
 
    // ----------------------------------------------
    // Define "A" block
    // ---------------------------------------------- 
    // number of elements in A
    const int NA = 7; 
    // number of columns in A 
    const int NACOL = 6;  
    // number of rows in A 
    const int NAROW = 2;  
    // Matrix elements - block A   
    double a[NA]  = { 1.,3.,1.,1.,1.,1.,1.}; 
    // Row indices - block A   
    int    ia[NA] = {0,1,0,0,0,1,1};
    // Column indices - block A   
    int    ja[NA] = {0,0,1,2,3,4,5};
    // ----------------------------------------------

    // Set first "A" block at position (0,0)  
    int blk = ekk_addElementBlock(test,NA,ia,ja,a);

    // Repeat "A" three times
    for (int iblk=1; iblk<4; iblk++) {
      
      // Compute row and column offset of repeated block
      int rowOff = iblk*NAROW;
      int colOff = iblk*NACOL;
      
      // Repeat block "A" making a new copy.   
      int ifCopy = 1;
      int blkout = ekk_repeatElementBlock(test,blk,rowOff,colOff,ifCopy);  
    }
 
    // ----------------------------------------------
    // Define "B" block
    // ----------------------------------------------
    // number of elements in B   
    const int NB = 5; 
    // number of columns in B  
    const int NBCOL = 6; 
    // Matrix elements - block B    
    double b[NB]  = { -4.,-1.,-10.,-1.,-1.}; 
    // Row indices - block B   
    int    ib[NB] = {2,2,3,0,1};
    // Column starts - block B   
    int    jb[NBCOL+1] = {0,1,3,4,4,4,5};     
    // ----------------------------------------------

    // Set first "B" block at position (0,2) which is
    // just below an "A" so the offset is (0,NAROW).
    blk=ekk_addColumnElementBlockWithOffsets(test,
      NBCOL,0,NAROW,ib,jb,b);

    // Repeat "B" three times
    for ( iblk=1; iblk<4; iblk++) {
      
      // Compute row and column offset of repeated block.
      // "B" block is placed just below A Block.
      int rowOff = iblk*NAROW+NAROW;
      int colOff = iblk*NACOL;      
      
      // Repeat block "B" without making a new copy.
      // The repeated blocks all occupy the same storage.  
      int ifCopy = 0;
      int blkout = ekk_repeatElementBlock(test,blk,rowOff,colOff,ifCopy);      
    }
 
    // ----------------------------------------------
    // Define "C" block
    // ----------------------------------------------  
    // number of elements in C  
    const int NC = 8;
    // Matrix elements - block C   
    double c[NC]  = { 1.,-1.,1.,1.,1.,-1.,1.,1. };
    // Row indices - block C   
    int    ic[NC] = { 0,2,0,1,1,3,2,3 };
    // Column indices - block C   
    int    jc[NC] = { 0,0,1,2,3,3,4,5 };
    // ----------------------------------------------
    
    // Set "C" block at position (8,24).  
    blk=ekk_addElementBlockWithOffsets(test,NC,ic,jc,c,8,24);    
    
    // Consolidate all the block sinto one block   
    rtcod = ekk_mergeBlocks(test,1);

    // Solve using simplex method   
    rtcod = ekk_primalSimplex(test,2);

    // Store model in MPS format   
    rtcod = ekk_exportModel(test,"Repeat.mps",1,2);

    // Set to print the nonzero column solutions only   
    rtcod = ekk_setIsolmask(test,6);

    // Print the solution   
    rtcod = ekk_printSolution(test);

    // Delete the model   
    rtcod = ekk_deleteModel(test);

    // OSL Destructor   
    ekk_endContext(env);
}
