// ---------------------------------------------------------------------
//                            EXSELNAM
//
// This program solves the following problem:
//
// maximize   x1 + 2x5 - x8
// subject to:
//
// 2.5 <=   3x1 +  x2          - 2x4 - x5              -    x8
//                2x2 + 1.1x3                                  <=  2.1
//                         x3             + x6                  =  4.0
// 1.8 <=                      2.8x4            -1.2x7         <=  5.0
// 3.0 <= 5.6x1                      + x5              + 1.9x8 <= 15.0
//
// and subject to:
//
// 2.5 <= x1
//   0 <= x2 <= 4.1
//   0 <= x3
//   0 <= x4
// 0.5 <= x5 <= 4.0
//   0 <= x6
//   0 <= x7
//   0 <= x8 <= 4.3
//
// The routines ekk_getCname, ekk_getCrhs, ekk_getCrange are used to
// set the problem name, the right hand side name and the range name.
// The routines ekk_copyRowNames and ekk_copyColumnNames are used to add
// row and column names. The routine ekk_selectionList is used restrict
// the ekk_printSolution and ekk_exportModel operations to rows and columns
// whose names match a specified character string.
//
// A secondary objective of this program is to illustrate how character
// string arguments can be passed between a C program and the routines
// ekk_copyRowNames and ekk_copyColumnNames.
// ---------------------------------------------------------------------

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include "ekk_c_api.h"

#define NCOL     8      // no. of columns in the constraint matrix   
#define NROW     5      // no. of rows in the constraint matrix   
#define NEL      14     // no. of elements in the constraint matrix   
#define ITYPE    2      // storage format: by columns   
#define IEL      14     // length of arrays containing element info.   
#define IRL      5      // length of arrays containing row info.   
#define ICL      8      // length of arrays containing column info.   
#define ICL1     9

char    probnam1[]="TPROB01";
char    rhs1[]="TRHS1";
char    range1[]="TRANGE1";


                               // Row indices   
int    mrow[IEL] = {0,4,0,1,1,2,0,3,0,4,2,3,0,4};

                               // Column starts   
int    mcol[ICL1]= {0,2,4,6,8,10,11,12,14};

                               // Matrix elements   
double  dels[IEL] = {3.0,5.6,1.0,2.0,1.1,1.0,-2.0,2.8,-1.0,
                     1.0,1.0,-1.2,-1.0,1.9};

                               // Lower bounds on row activities   
double  drlo[IRL] = {2.5,-1.0e+31,4.0,1.8,3.0};

                               // Upper bounds on row activities   
double  drup[IRL] = {1.0e+31,2.1,4.0,5.0,1.5e+1};

                               // Lower bounds on columns   
double  dclo[ICL] = {2.5,0.0,0.0,0.0,0.5,0.0,0.0,0.0};

                               // Upper bounds on columns   
double  dcup[ICL] = {1.0e+31,4.1,1.0e+31,1.0e+31,4.0,1.0e+31,
                     1.0e+31,4.3};

                               // Objective function coefficients   
double  dobj[ICL] = {1.0,0.0,0.0,0.0,2.0,0.0,0.0,-1.0};


void main()
{
    static char  selname1[]="?C*";

    // Initialize OSL environment   
    EKKContext * env;
    env = ekk_initializeContext();

    // Create an OSL model "test"   
    EKKModel * test;
    test = ekk_newModel(env,NULL);

    // Set to maximization problem   
    int rtcod = ekk_setRmaxmin(test,-1.000000);

    ekk_setPrinting(test,1,1,0);

    // Define the rim vectors   
    rtcod = ekk_loadRimModel(test,NROW,drlo,drup,NCOL,dobj,dclo,dcup);

    // Define the matrix elements   
    rtcod = ekk_addColumnElementBlock(test,NCOL,mrow,mcol,dels);
    rtcod = ekk_mergeBlocks(test,1);

    // Set the problem name   
    rtcod = ekk_setCname(test,probnam1);

    // Set the RHS block name   
    rtcod = ekk_setCrhs(test,rhs1);

    // Set the RANGE block name   
    rtcod = ekk_setCrange(test,range1);
    char  *colnames[]={"TCOL001","TCOL002","TCOL003",
                 "TCOL004","TCOL005","TCOL006","TCOL007","TCOL008"};
    char  *rownames[]={"TROW01","TROW02","TROW03","TROW04","TROW05"};

    // Add user row names to model   
    rtcod = ekk_copyRowNames(test,rownames,0,5);

    // Add user column names to model   
    rtcod = ekk_copyColumnNames(test,colnames,0,8);

    // Write model to file in MPS format   
    rtcod = ekk_exportModel(test,"wholeMatrix",1,2);

    // Solve using simplex method   
    rtcod = ekk_primalSimplex(test,2);

    // Print the solution   
    rtcod = ekk_printSolution(test);

    rtcod = ekk_selectionList(test,selname1,'*','?',1,1);

    // Print the solution   
    rtcod = ekk_printSolution(test);

    // Write model to file in MPS format   
    rtcod = ekk_exportModel(test,"selMatrix",1,2);

    // Delete the model   
    rtcod = ekk_deleteModel(test);

    // OSL Destructor   
    ekk_endContext(env);
}
