// **********************************************************************
//
//                           EXSETBAS
//
//  This program reads a maximization problem from an MPS file, sets
//  the appropriate control variable to indicate that it is a
//  maximization problem, solves the problem with the simplex method,
//  and stores the basic variables in a user defined array for use
//  for a second model.
//
// **********************************************************************

#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include "ekk_c_api.h"

void main()
{
    // Initialize OSL environment
    EKKContext * env;
    env = ekk_initializeContext();

    // Create an OSL model "test"
    EKKModel * test;
    test = ekk_newModel(env,NULL);

    // Set to maximization problem
    int rtcod = ekk_setRmaxmin(test,-1.000000);

    // Read model data from MPS file
    rtcod = ekk_importModel(test,"p0033.mps");

    // Scale the model data
    rtcod = ekk_scale(test);

    // Solve using simplex method
    rtcod = ekk_primalSimplex(test,1);

    // Set to print only the statistics
    rtcod = ekk_setIprtinfomask(test,2);

    // Print the solution
    rtcod = ekk_printSolution(test);

    // Get the iteration count
    int iter1 = ekk_getIiternum(test);

    // Get the number of columns
    int numCols = ekk_getInumcols(test);

    // Get the number of rows
    int numRows = ekk_getInumrows(test);

    // Set up user array for storing column status
    int * usrColStat = ekk_int(numCols);

    // Set up user array for storing column indices
    int * usrColNum  = ekk_int(numCols);

    // Set up user array for storing row status
    int * usrRowStat = ekk_int(numRows);

    // Set up user array for storing row indices
    int * usrRowNum  = ekk_int(numRows);

    // Get address of column status region
    const int * colstat = ekk_colstat(test);

    // Get address of row status region
    const int * rowstat = ekk_rowstat(test);

    for (int ic = 0; ic<numCols; ic++) {
        // store the column indices
        usrColNum[ic] = ic;

        // store the column status
        usrColStat[ic] = colstat[ic];
    }
    for (int ir = 0; ir < numRows; ir++) {
        // store the row indices
        usrRowNum[ir] = ir;
        // store the row status
        usrRowStat[ir] = rowstat[ir];
    }
    // Delete the model
    rtcod = ekk_deleteModel(test);

    // Create a second OSL model "test2"
    EKKModel * test2;
    test2 = ekk_newModel(env,NULL);

    // Set to maximization problem
    rtcod = ekk_setRmaxmin(test2,-1.000000);

    // Read model data from MPS file
    rtcod = ekk_importModel(test2,"p0033.mps");

    // Set the status of columns
    rtcod = ekk_setColumnStatus(test2,numCols,usrColNum,usrColStat);

    // Set the status of rows
    rtcod = ekk_setRowStatus(test2,numRows,usrRowNum,usrRowStat);

    // Solve using simplex method
    rtcod = ekk_primalSimplex(test2,0);

    // Set to print only the statistics
    rtcod = ekk_setIprtinfomask(test2,2);

    // Print the solution
    rtcod = ekk_printSolution(test2);

    // Get the iteration count
    int iter2 = ekk_getIiternum(test2);

    printf("\n\n Iterations for the original (cold-start) solve = %d",iter1);
    printf("\n Iterations for the hot-start (with known basic rows/columns) solve = %d\n\n",iter2);

    // OSL Destructor
    ekk_endContext(env);
}
