// ---------------------------------------------------------------------
//                             EXSTATUS
// ---------------------------------------------------------------------
//
// This program solves the following problem:
//
// maximize   x0 +  x1 + 9x2 + 6x3 +  x4 +  x5 +  x6
// subject to:
//
//           -x0        - x2                           = -2
//            x0  - x1        - x3                     =  0
//                  x1              - x4               =  0 
//                                    x4        - x6   =  0
//                              x3        - x5    x6   =  0
//                        x2                x5         =  2
//
// and subject to:
//
//   0 <= x0
//   0 <= x1
//   0 <= x2
//   0 <= x3
//   0 <= x4 <= 1
//   0 <= x5
//   0 <= x6
//
//   This is a pure network problem with a unique optimal solution 
//   given by:    x0=2, x1=1, x2=0, x3=1, x4=1, x5=2, x6=1 
//
//   All variables are basic, except x2 which is non-basic at lower
//   bound and x4 which is non-basic at upper bound.  This leads to
//   a characterization of an optimal column status region.  A 
//   corresponding optimal row status region would typically involve
//   one basic row, and five rows that are non-basic at lower bound. 
//   (To say that a row is basic is to say that the accompanying slack 
//   variable is basic.)
//
//   The driver below initializes optimal column and row status regions
//   and then verifies optimality by calling ekk_simplex().  Notice
//   that the simplex solver uses only one pivot to verify optimality.
//
// --------------------------------------------------------------------- 

#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include "ekk_c_api.h"

#define MAXSPC   50000
#define NCOL     7      // no. of columns in the constraint matrix           
#define NROW     6      // no. of rows in the constraint matrix              
#define NEL      14     // no. of elements in the constraint matrix          
#define ITYPE    2      // storage format: by columns                        
#define IEL      14     // length of arrays containing element info.         
#define IRL      6      // length of arrays containing row info.             
#define ICL      7      // length of arrays containing column info.          
#define ICL1     8 

                               // Row indices
int     mrow[IEL] = {1,0,2,1,5,0,4,1,3,2,5,4,4,3};

                               // Column starts
int     mcol[ICL1]= {0,  2,  4,  6,  8,  10, 12, 14};

                               // Matrix elements
double  dels[IEL] = {1.0,-1.0,1.0,-1.0,1.0,-1.0,1.0,-1.0,
                     1.0,-1.0,1.0,-1.0,1.0,-1.0};

                               // Lower bounds on row activities
double  drlo[IRL] = {-2.0,0.0,0.0,0.0,0.0,2.0};

                               // Upper bounds on row activities
double  drup[IRL] = {-2.0,0.0,0.0,0.0,0.0,2.0};

                               // Lower bounds on columns
double  dclo[ICL] = {0.0,0.0,0.0,0.0,0.0,0.0,0.0};

                               // Upper bounds on columns
double  dcup[ICL] = {10.0,10.0,10.0,10.0,1.0,10.0,10.0};

                               // Objective function coefficients
double  dobj[ICL] = {1.0,1.0,9.0,6.0,1.0,1.0,1.0};

void main()
{   int rtcod;

    // Initialize OSL environment   
    EKKContext * env;
    env = ekk_initializeContext();

    // Create an OSL model "test"   
    EKKModel * test;
    test = ekk_newModel(env,NULL);

    // Set to maximization problem  
    // rtcod = ekk_setRmaxmin(test,-1.000000);

    // Set to print parameter values upon entry to OSL functions  
    ekk_setPrinting(test,1,1,0);

    // Load row/column bounds, obj function
    rtcod = ekk_loadRimModel(test,NROW,drlo,drup,NCOL,dobj,dclo,dcup);

    // Load the matrix elements by column
    rtcod = ekk_addColumnElementBlock(test,NCOL,mrow,mcol,dels);

    // Merge the blocks into 1 block
    rtcod = ekk_mergeBlocks(test,1);

    // Get row status region
    int XBASIC = 1 << 31;
    int XDOWN  = 1 << 30;  /* This row or column may go down. (It is
                              non-basic at upper bound.) */
    int XUP    = 1 << 29;  /* This row or column may go up. (It is
                              non-basic at lower bound.) */ 
    int *rstat = ekk_getRowstat(test);
    int *cstat = ekk_getColstat(test);

    rstat[0] = XBASIC; rstat[1] = XUP; rstat[2] = XUP; 
    rstat[3] = XUP;    rstat[4] = XUP; rstat[5] = XUP;

    cstat[0] = XBASIC; cstat[1] = XBASIC; cstat[2] = XUP; cstat[3] = XBASIC;
    cstat[4] = XDOWN;  cstat[5] = XBASIC; cstat[6] = XBASIC;

    rtcod = ekk_setRowstat(test,rstat);
    rtcod = ekk_setColstat(test,cstat);

    // Solve using simplex method   
    rtcod = ekk_primalSimplex(test,2);

    // Print the solution           
    rtcod = ekk_printSolution(test);

    // Delete the model             
    rtcod = ekk_deleteModel(test);

    // OSL Destructor               
    ekk_endContext(env);

}
